@@ -1,3 +1,14 @@
+# Standardize the left-hand side of a formula, i.e., get a slightly modified
+# formula as well as the response variable names before and after any
+# evaluations of special expressions:
+stdize_lhs <- function(formul_crr) {
+  formul_crr <- rm_cbind(formul_crr)
+  formul_crr <- rm_addresp(formul_crr)
+  y_nm_orig <- as.character(formul_crr)[2]
+  y_nm <- gsub("\\(|\\)", "", y_nm_orig)
+  return(nlist(fml = formul_crr, y_nm_orig, y_nm))
+}
+
 # A function to retrieve the formula from a fit (`fit_obj`):
 get_formul_from_fit <- function(fit_obj) {
   formul_out <- formula(fit_obj)
@@ -11,11 +22,9 @@ get_formul_from_fit <- function(fit_obj) {
 # (`formul_crr`):
 get_dat_formul <- function(formul_crr, needs_adj, dat_crr = dat) {
   if (needs_adj) {
-    formul_crr <- rm_cbind(formul_crr)
-    formul_crr <- rm_addresp(formul_crr)
-    y_nm_orig <- as.character(formul_crr)[2]
-    y_nm <- gsub("\\(|\\)", "", y_nm_orig)
-    dat_crr[[y_nm]] <- eval(str2lang(y_nm_orig), dat_crr)
+    stdized_lhs <- stdize_lhs(formul_crr)
+    dat_crr[[stdized_lhs$y_nm]] <- eval(str2lang(stdized_lhs$y_nm_orig),
+                                        dat_crr)
   }
   return(dat_crr)
 }
